/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.jms.listener;

import java.util.logging.Level;

import javax.jms.JMSException;
import javax.jms.MessageConsumer;

import org.ow2.petals.bc.jms.connection.JMSConsumerConnection;
import org.ow2.petals.bc.jms.connection.JMSConsumerConnectionFactory;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;

public final class JMSExternalListener extends AbstractExternalListener {

    private JMSConsumerConnectionFactory connectionFactory;

    private JMSConsumerConnection connection;

    @Override
    public void init() {
        try {
            this.createConnectionFactory();
        } catch (final PEtALSCDKException e) {
            this.getLogger().log(Level.SEVERE, "Can't create consumer connection factory", e);
        }
    }

    private void createConnectionFactory() throws PEtALSCDKException {
        this.connectionFactory = new JMSConsumerConnectionFactory(this.getExtensions(), component
                .getLogger());
    }

    @Override
    public void start() throws PEtALSCDKException {
        try {
            this.connection = (JMSConsumerConnection) this.connectionFactory.makeObject();
            this.connectionFactory.activateObject(this.connection);
        } catch (final Exception e1) {
            this.getLogger().log(Level.SEVERE,
                    "Can not retrieve a JMS connection for consumes : " + this.getConsumes(), e1);
        }

        if (connection != null) {
            final JMSMessageListener jmsMessageListener = new JMSMessageListener(this);
            final MessageConsumer consumer = this.connection.getMessageConsumer();
            try {
                if (consumer != null) {
                    consumer.setMessageListener(jmsMessageListener);
                }
            } catch (final JMSException e) {
                throw new PEtALSCDKException(
                        "Can not set the JMS messageListener for the JMS consumer.", e);
            }
        }

    }

    @Override
    public void stop() throws PEtALSCDKException {
        try {
            if (this.connection != null) {
                connectionFactory.passivateObject(this.connection);
                connectionFactory.destroyObject(this.connection);
            }
        } catch (final Exception e) {
            throw new PEtALSCDKException("Can not destroy a JMS connection for consumes : "
                    + getConsumes(), e);
        }
    }

}
