/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.jms.su;

import javax.jbi.servicedesc.ServiceEndpoint;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.ow2.petals.bc.jms.JmsBC;
import org.ow2.petals.bc.jms.connection.JMSProviderConnectionFactory;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.BindingComponentServiceUnitManager;

import static org.ow2.petals.bc.jms.Constants.Extensions.MAX_ACTIVE;
import static org.ow2.petals.bc.jms.Constants.Extensions.MAX_IDLE;
import static org.ow2.petals.bc.jms.Constants.Extensions.MAX_WAIT;
import static org.ow2.petals.bc.jms.Constants.Extensions.MIN_EVICTABLE_IDLE_TIME_MILLIS;
import static org.ow2.petals.bc.jms.Constants.Extensions.TEST_WHILE_IDLE;
import static org.ow2.petals.bc.jms.Constants.Extensions.TIME_BETWEEN_EVICTION_RUNS_MILLIS;

public class JmsServiceUnitManager extends BindingComponentServiceUnitManager {

    private final JmsBC component;

    public JmsServiceUnitManager(JmsBC component) {
        super(component);
        this.component = component;
    }

    @Override
    public void doDeploy(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {
        for (final Provides provides : descriptor.getServices().getProvides()) {
            try {
                this.registerConnectionPool(provides);
            } catch (final PEtALSCDKException e) {
                throw new PEtALSCDKException("Error creating a connection pool for SU: "
                        + suRootPath, e);
            }
        }
    }

    @Override
    public void doStart(String serviceUnitName) throws PEtALSCDKException {
        // TODO Create some connections here

    }

    @Override
    public void doStop(String serviceUnitName) throws PEtALSCDKException {
        // TODO remove all connections from the pool ?

    }

    @Override
    public void doUndeploy(String serviceUnitName) throws PEtALSCDKException {
        for (final ServiceEndpoint endpoint : this.getEndpointsForServiceUnit(serviceUnitName)) {
            final Provides provides = this.getProvidesFromEndpoint(endpoint);
            try {
                this.unregisterConnectionPool(provides);
            } catch (final PEtALSCDKException e) {
                throw new PEtALSCDKException(e.getMessage(), e);
            }
        }
    }

    /**
     * Register a pool of JMS connections.
     * 
     * @param provides
     * @throws PEtALSCDKException
     */
    private void registerConnectionPool(Provides provides) throws PEtALSCDKException {

        final ConfigurationExtensions extensions = new ConfigurationExtensions(provides.getAny());
        if (extensions != null) {
            final int maxActive = Integer.parseInt(extensions.get(MAX_ACTIVE, "10"));
            final long maxWait = Long.parseLong(extensions.get(MAX_WAIT, "10000"));
            final int maxIdle = Integer.parseInt(extensions.get(MAX_IDLE, "5"));
            final long timeBetweenEvictionTimeMillis = Long.parseLong(extensions.get(
                    TIME_BETWEEN_EVICTION_RUNS_MILLIS, "30000"));
            final long minEvictableIdleTimeMillis = Long.parseLong(extensions.get(
                    MIN_EVICTABLE_IDLE_TIME_MILLIS, "600000"));
            final boolean testWhileIdle = Boolean.parseBoolean(extensions.get(TEST_WHILE_IDLE,
                    "true"));

            this.component.getJmsProducerConnections().put(
                    provides,
                    new GenericObjectPool(new JMSProviderConnectionFactory(extensions,
                            this.component.getLogger()), maxActive,
                            GenericObjectPool.WHEN_EXHAUSTED_BLOCK, maxWait, maxIdle, false, false,
                            timeBetweenEvictionTimeMillis, 5, minEvictableIdleTimeMillis,
                            testWhileIdle));
        }
    }

    /**
     * unRegister a pool of connections.
     * 
     * @param provides
     * @throws PEtALSCDKException
     */
    private void unregisterConnectionPool(Provides provides) throws PEtALSCDKException {
        // close pool
        final GenericObjectPool pool = this.component.getJmsProducerConnections().get(provides);

        try {
            pool.close();
        } catch (final Exception e) {
            throw new PEtALSCDKException("Can't close pool associated to provides node : "
                    + provides);
        }
        // remove pool
        this.component.getJmsProducerConnections().remove(provides);
    }
}
